/*
 * hash.c            -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "hash.h"
#include "maker.h"		/* for find_path_of_file		    */
#include "vars.h"
/**/
/**/
int	hash_function(char  *str)
{
	int	rv = 0;
	if ( str == NULL )
		return HASH_SIZE;
	while ( *str )
	{
		rv += *str;
		str++;
	}
	return rv % HASH_SIZE;
}
/**/
int	hash_create_element(	char		*file_name,
				HASH_TABLE	*h,
				HASH_ELEMENT 	**h_el,
				unsigned short	type)
{
	if ( ( h_el == NULL ) || ( strlen( file_name ) <= 0 ) )
		return BAD_PARAMETERS;

	switch ( type )
	{
	case LOCAL_HASH_TABLE:
		if ( h->allocation >= HASH_MAX_STATIC_ALLOCATION )
		{
			printf(	"\nCannot allocate memory for LOCAL_HASH_TABLE, see File %s, Line %d.",
				__FILE__,
				__LINE__ );
			return MEMORY_ALLOCATION_FAILURE;
		}
		*h_el = &( local_static_allocation[ (h->allocation)++ ] );
		break;

	case GLOBAL_HASH_TABLE:
		if ( h->allocation >= HASH_MAX_STATIC_ALLOCATION )
		{
			printf(	"\nCannot allocate memory for GLOBAL_HASH_TABLE, see File %s, Line %d.",
				__FILE__,
				__LINE__ );
			return MEMORY_ALLOCATION_FAILURE;
		}
		*h_el = &( global_static_allocation[ (h->allocation)++ ] );
		break;
	}

	strcpy( (*h_el)->filename , file_name );
	( *h_el )->next	= NULL;

	switch ( type )
	{
	case LOCAL_HASH_TABLE:
		( *h_el )->rest.local.visited = ( *h_el )->rest.local.openned = False;
		( *h_el )->rest.local.full_filename_ptr = NULL;
		break;
	case GLOBAL_HASH_TABLE:
		( *h_el )->rest.global.full_filename = PATH_INDEX_NONE;
		break;
	default:
		break;
	}

	return	MMAKER_OK;
}
/**/
int	hash_init(	HASH_TABLE	*h,
			unsigned int	type)
{
  	int	i;

	if (	( h == NULL ) ||
		( ( type != GLOBAL_HASH_TABLE ) &&
		  ( type != LOCAL_HASH_TABLE  )	)	)
		return BAD_PARAMETERS;
	for ( i = 0 ; i < HASH_SIZE ; i++ )
		h->bucket[i] = NULL;
	h->type = type;
	h->allocation = 0;

	return	MMAKER_OK;
}
/**/
int	hash_is_member_file_name(HASH_TABLE	*h,
				 char		*file_name)
{
	int		bucket_num;
	HASH_ELEMENT	*e_tmp;

	if ( ( h == NULL ) || ( file_name == NULL ) )
		return BAD_PARAMETERS;

	bucket_num = hash_function( file_name );
	e_tmp = h->bucket[ bucket_num ];
	while ( e_tmp != NULL )
	{
		if ( strcmp( e_tmp->filename, file_name )  != 0 )
			e_tmp = e_tmp->next;
		else
			return	ELEMENT_PRESENT;
	}
	return	ELEMENT_NOT_PRESENT;
}
/**/
HASH_ELEMENT  *hash_is_member_file_name_ptr(	HASH_TABLE	*h,
						char		*file_name)
{
	int		bucket_num;
	HASH_ELEMENT	*e_tmp;

	if ( ( h == NULL ) || ( file_name == NULL ) )
		return NULL;

	bucket_num = hash_function( file_name );
	e_tmp = h->bucket[ bucket_num ];
	while ( e_tmp != NULL )
	{
		if ( strcmp( e_tmp->filename, file_name )  != 0 )
			e_tmp = e_tmp->next;
		else
			return	e_tmp;
	}
	return	NULL;
}
/**/
int	hash_insert_file_name(	HASH_TABLE	*h,
				char		*file_name)
{
	HASH_ELEMENT	*e;
	int		bucket_num;
	HASH_ELEMENT	*old_e;

	if ( ( h == NULL ) || ( file_name == NULL ) )
		return BAD_PARAMETERS;

	if ( hash_is_member_file_name(h,file_name) == ELEMENT_PRESENT )
		return	ELEMENT_PRESENT;

	if ( hash_create_element(file_name,
				 h,
				 &e,
				 h->type) != MMAKER_OK )
		return	ELEMENT_NOT_PRESENT;

	bucket_num = hash_function( file_name );
	if ( bucket_num == HASH_SIZE )
		return ELEMENT_NOT_PRESENT;

	if ( h->type == GLOBAL_HASH_TABLE )
		e->rest.global.full_filename = find_path_index_of_file( file_name );

	old_e = h->bucket[ bucket_num ];
	h->bucket[ bucket_num ] = e;
	e->next = old_e;
	h->hash_index = 0;
	return	ELEMENT_PRESENT;
}
/**/
int	hash_print( HASH_TABLE	*h )
{
	int		i;
	HASH_ELEMENT 	*e;

	if ( h == NULL )
		return BAD_PARAMETERS;

	switch ( h->type )
	{
	case GLOBAL_HASH_TABLE:
		printf("\nGLOBAL HASH TABLE :");
		break;
	case LOCAL_HASH_TABLE:
		printf("\nLOCAL HASH TABLE :");
		break;
	default:
		printf("\nUNKNOWN HASH TABLE :");
		break;
	}

	for (i = 0; i < HASH_SIZE; i++)
	{
		e = h->bucket[i];
		printf("\n\nBucket # %d",i);
		printf("\n=====================");
		while (e)
		{
			printf("\n%s",			e->filename);
			switch ( ( *h ).type )
			{
			case GLOBAL_HASH_TABLE:
				printf("\n\t%d", e->rest.global.full_filename);
				break;
			case LOCAL_HASH_TABLE:
				printf("\n\tvisited\t%s",
					(e->rest.local.visited ? "VISITED" : "NOT VISITED"));
				printf("\n\topenned\t%s",
					(e->rest.local.openned ? "OPENNED" : "NOT OPENNED"));
				printf("\n\t%s", e->rest.local.full_filename_ptr);
				break;
			default:
				break;
			}

			printf("\n\tnext\t%p",		(void *)e->next);

			e = e->next;
		}
	}
	return	MMAKER_OK;
}
/**/
int	hash_index_init( HASH_TABLE	*h )
{
	if (h == NULL)
		return	BAD_PARAMETERS;
	h->hash_index = 0;
	return	MMAKER_OK;
}
/**/
HASH_ELEMENT  *hash_next( HASH_TABLE	*h )
{
	if (	( h == NULL )				||
		( (h->type != GLOBAL_HASH_TABLE )	&&
		  (h->type != LOCAL_HASH_TABLE  ) )	)
		return NULL;

	if (	( h->allocation > 0 )	&&
		( h->hash_index != h->allocation )	)
	{
		if ( h->type == GLOBAL_HASH_TABLE )
			return &global_static_allocation[ h->hash_index++ ];
		else
			return &local_static_allocation[ h->hash_index++ ];
	}
	else
	{
		h->hash_index = 0;
		return	NULL;
	}
}
/**/
void 	*hash_next_unvisited( HASH_TABLE	*h )
{
	HASH_ELEMENT	*tmp_el;

	if ( (h == NULL) || (h->type != LOCAL_HASH_TABLE) )
		return NULL;

	tmp_el = (HASH_ELEMENT  *)hash_next( h );
	while ((tmp_el != NULL) && (tmp_el->rest.local.visited == True))
		tmp_el = (HASH_ELEMENT  *)hash_next( h );

	return	tmp_el;
}

/**/
int	hash_print_element(	HASH_ELEMENT 	*e,
				unsigned int	type )
{
	if (e == NULL)
		return	BAD_PARAMETERS;

	printf("\n\nBucket # %d", hash_function(e->filename) );
	printf("\n=====================");

	printf("\n%s", e->filename);

	switch ( type )
	{
	case GLOBAL_HASH_TABLE:
		printf("\n\t%d", e->rest.global.full_filename);
		break;
	case LOCAL_HASH_TABLE:
		printf("\n\tvisited\t%s",
			(e->rest.local.visited ? "VISITED" : "NOT VISITED"));
		printf("\n\topenned\t%s",
			(e->rest.local.openned ? "OPENNED" : "NOT OPENNED"));
		printf("\n\t%s", e->rest.local.full_filename_ptr);
		break;
	default:
		printf("\n\tERROR : UNKNOWN HASH TABLE TYPE");
		break;
	}

	printf("\n\tnext\t%p\n\n",	(void *)e->next);
	return	MMAKER_OK;
}
/**/
int	hash_are_all_visited( HASH_TABLE	*h )
{
	unsigned short		i;

	if ( (h == NULL) || (h->type != LOCAL_HASH_TABLE) )
		return	BAD_PARAMETERS;

	for ( i = 0; i < h->allocation; i++ )
		if ( h->bucket[ i ]->rest.local.visited != True )
			return False;

	return	True;
}
/**/
int	hash_size( HASH_TABLE	*h )
{
	if (h == NULL)
		return	BAD_PARAMETERS;

	return	h->allocation;
}
/**/
