/*
 * hash.h            -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

#ifndef HASH__H
#define HASH__H

#include "globals.h"
/*--------------------------------------------------------------------------*/
#ifndef HASH_MAX_STATIC_ALLOCATION
#error  HASH_MAX_STATIC_ALLOCATION  is undefined!
#endif
/*--------------------------------------------------------------------------*/
#define	ELEMENT_PRESENT				0
#define	ELEMENT_NOT_PRESENT			1
#define LOCAL_HASH_TABLE			1
#define GLOBAL_HASH_TABLE			2
#define HASH_SIZE                       	91
/*--------------------------------------------------------------------------*/
typedef enum { False, True }    BOOLEAN;
/*--------------------------------------------------------------------------*/
typedef union HASH_REST_
{
	struct
	{
		BOOLEAN	visited;
		BOOLEAN openned;
		int	*full_filename_ptr;
	}	local;

	struct
	{
		int	full_filename;
	}	global;
}	HASH_REST;
/*--------------------------------------------------------------------------*/
typedef struct HASH_ELEMENT_
{
	char                    filename[ _MAX_PATH ];
	struct HASH_ELEMENT_	*next;
	HASH_REST		rest;
}       HASH_ELEMENT ;
/*--------------------------------------------------------------------------*/
typedef struct
{
	unsigned short	hash_index;
	HASH_ELEMENT	*bucket[ HASH_SIZE ];
	unsigned short	type;
	unsigned short	allocation;
}       HASH_TABLE;
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/* #----------------------------------------------------------------------# */
/* |    General Note                                                      | */
/* |    All routines will return 0 on success                             | */
/* #----------------------------------------------------------------------# */

int     	hash_function( char 	*str );
int		hash_create_element(	char		*file_name,
					HASH_TABLE	*h,
					HASH_ELEMENT 	**h_el,
					unsigned short	type);
int     	hash_init(      HASH_TABLE      *h,
				unsigned int	type);

int     	hash_is_member_file_name(HASH_TABLE	*h,
			 		 char		*file_name);
HASH_ELEMENT	*hash_is_member_file_name_ptr(	HASH_TABLE      *h,
						char		*file_name);
int     	hash_insert_file_name(  HASH_TABLE	*h,
					char		*file_name);
int     	hash_print(     HASH_TABLE      *h);
int     	hash_index_init(HASH_TABLE      *h);
HASH_ELEMENT	*hash_next(	HASH_TABLE	*h);
void 		*hash_next_unvisited(   HASH_TABLE      *h);
int     	hash_print_element(     HASH_ELEMENT	*e,
					unsigned int	type);
int     	hash_are_all_visited(   HASH_TABLE	*h);
int     	hash_size( HASH_TABLE   *h);
/*--------------------------------------------------------------------------*/
#endif          /* #ifndef      HASH__H                                     */
