/*
 * macros.c          -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

#include <stdio.h>		/* for printf()				    */
#include <string.h>		/* for strcpy()				    */
#include <stdlib.h>		/* for _MAX_PATH			    */
#include "macros.h"		/* for routine prototypes		    */
#include "globals.h"		/* for PATH_SEPERATOR_CHAR		    */
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
int	macros_match(	char		*pattern,
			char		*str,
			unsigned int	max_len	)
{
	if ( max_len == 0 )
		return 0;

	if ( pattern == NULL )
		return 0;

	if ( pattern[ 0 ] == '\0' )
		return 0;

	if ( str == NULL )
		return 1;

	if ( str[ 0 ] == '\0' )
		return 1;

	if ( str[ 0 ] != pattern[ 0 ] )
		return 1;

	return ( macros_match( pattern + 1, str + 1, max_len - 1 ) );
}
/*--------------------------------------------------------------------------*/
#define EXPAND(pattern,variable)							\
	if ( macros_match( pattern, str_in + i, strlen(pattern) ) == 0 )			\
	{										\
		if ( ( i != 0 )			&&					\
		     ( str_in[ i - 1 ] == '\\' ))					\
		{									\
			i++;								\
			strncpy( str_out + j, str_in + i, strlen(pattern) - 1 );	\
			j += (strlen(pattern) - 1);					\
			i += (strlen(pattern) - 1);					\
		}									\
		else									\
		{									\
			strcat( str_out + j, variable );				\
			j += strlen( variable );					\
			i += strlen(pattern);						\
		}									\
	}
/*--------------------------------------------------------------------------*/
void	macros_expand_compile_macros(	char		*str_in,
					char		*str_out,
					COMPILE_MACROS	*mptr	)
{
	int	i = 0,
		j = 0;


	while ( str_in[ i ] != '\0' )
	{
		while ( ( str_in[ i ] != '\0' ) && ( str_in[ i ] != '\\' ) )
		{
			str_out[ j ] = str_in[ i ];
			i++;
			j++;
			str_out[ j ] = '\0';
		}
		if ( str_in[ i ] == '\0' )
			break;
		EXPAND(MACROS_FILENAME_PATTERN,mptr->filename)
		else
		EXPAND(MACROS_FILENAME1_PATTERN,mptr->filename)
		else
		EXPAND(MACROS_BASENAME_PATTERN,mptr->basename)
		else
		EXPAND(MACROS_BASENAME1_PATTERN,mptr->basename)
		else
		EXPAND(MACROS_EXTENTION_PATTERN,mptr->extention)
		else
		EXPAND(MACROS_EXTENTION1_PATTERN,mptr->extention)
		else
		EXPAND(MACROS_NEWLINE_PATTERN,mptr->newline)
		else
		EXPAND(MACROS_NEWLINE1_PATTERN,mptr->newline)
		else
		EXPAND(MACROS_BACKSLASH_PATTERN,"\\")
		else
		EXPAND(MACROS_BACKSLASH1_PATTERN,"\\")
		else
		{
			str_out[ j ] = str_in[ i ];
			i++;
			j++;
		}
		str_out[ j ] = '\0';
	}
}
/*--------------------------------------------------------------------------*/
int	macros_compute_compile_macros(	char		*filename,
					COMPILE_MACROS	*mptr	)
{
	char	*basename_ptr;
	char	*name_ptr;

	if ( mptr == NULL )
		return 1;
	
	basename_ptr = mptr->basename;
	strcpy( mptr->filename,  filename );
	name_ptr = filename + strlen( filename ) - 1;
	while ( ( name_ptr > filename )			&&
		( name_ptr[-1] != PATH_SEPERATOR_CHAR )	)
		name_ptr--;
	strcpy( mptr->basename,  name_ptr );
	while ( *basename_ptr != '.' )
		basename_ptr++;
	*basename_ptr = '\0';
	strcpy( mptr->extention, basename_ptr + 1 );
	strcpy( mptr->newline, "\n\t" );
	return 0;
}
/*--------------------------------------------------------------------------*/
int COUNT(char *pattern, int *i, char *str_in, int *count)									\
{
	if ( macros_match( pattern, str_in + *i, strlen(pattern) ) == 0 )		\
	{										\
		if ( str_in[ *i - 1 ] != '\\' )						\
		{									\
			(*count)++;							\
			*i += strlen(pattern);						\
		}									\
		else									\
			(*i)++;								\
		return 1;
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
int	macros_expandable_in_string_count( char	*str_in )
{
	int	count = 0;
	int	i = 0;


	while ( str_in[ i ] != '\0' )
	{
		while ( ( str_in[ i ] != '\0' ) && ( str_in[ i ] != '\\' ) )
		{
			i++;
		}
		if ( str_in[ i ] == '\0' )
			break;
		if ( COUNT( MACROS_FILENAME_PATTERN, &i, str_in, &count) == 0 )
		   if ( COUNT( MACROS_FILENAME1_PATTERN, &i, str_in, &count) == 0 )
		      if ( COUNT( MACROS_BASENAME_PATTERN, &i, str_in, &count) == 0 )
		         if ( COUNT( MACROS_BASENAME1_PATTERN, &i, str_in, &count) == 0 )
		            if ( COUNT( MACROS_EXTENTION_PATTERN, &i, str_in, &count) == 0 )
		               if ( COUNT( MACROS_EXTENTION1_PATTERN, &i, str_in, &count) == 0 )
			          if ( COUNT( MACROS_NEWLINE_PATTERN, &i, str_in, &count) == 0 )
			             if ( COUNT( MACROS_NEWLINE1_PATTERN, &i, str_in, &count) == 0 )
			                if ( COUNT( MACROS_BACKSLASH_PATTERN, &i, str_in, &count) == 0 )
			                   if ( COUNT( MACROS_BACKSLASH1_PATTERN, &i, str_in, &count) == 0 )
				              i++;
	}
	return	count;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
