/*
 * options.c         -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

/*--------------------------------------------------------------------------*/
#include "globals.h"
/*--------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dirent.h>

/*--------------------------------------------------------------------------*/

#ifndef	GNUC_LINUX_VERSION
#include <conio.h>
#include <direct.h>
#include <dos.h>
#else
#include <time.h>
#include <sys/types.h>
#endif

#include "options.h"
#include "hash.h"
#include "maker.h"
#include "vars.h"
#include "xxmacros.h"
#include "xxpathes.h"
#include "file_arg.h"
/*--------------------------------------------------------------------------*/
	char	options_save_setup_filename	[ _MAX_PATH ]	= "";
	char	options_spc_obj_dir		[ _MAX_PATH ]	= "obj";
	char	options_spc_target_dir		[ _MAX_PATH ]	= "";
	char	options_spc_run_linker_dir	[ _MAX_PATH ]	= "";
	char	options_spc_extention		[ 4 ] 		= 
#ifdef	__MSDOS__
	"lib";
#else
	"";
#endif
/*--------------------------------------------------------------------------*/
void	options_init_extra_extentions( void )
{
  	unsigned short	i;

	for ( i = 0; i < 10; i++ )
	  	extra_extentions[ i ].target[ 0 ] = extra_extentions[ i ].source[ 0 ] = extra_extentions[ i ].command_line[ 0 ] = '\0';
}
/*--------------------------------------------------------------------------*/
void	options_interpret_extra_extentions( char *extra_line )
{
	unsigned short	extra_entry;
	unsigned short	extra_len1;
	unsigned short	extra_len2;
	unsigned short	extra_len3;
	char		*cptr;
	char		*extra_ext1;
	char		*extra_ext2;
	char		*extra_action;

	/* 								    */
	/* Requested format:						    */
	/* 	[0-9]{XXX}.{YYY}.{Free TEXT}				    */
	/* 								    */

	cptr = extra_line;

	if ( (*cptr < '0') || (*cptr > '9') )
	{
	  	printf("\nNot a legal line: -X%s", extra_line);
		return;
	}

	extra_entry = *cptr - '0';
	cptr++;
	extra_ext1 = strtok( cptr, "." );
	extra_ext2 = strtok( NULL, "." );
	extra_len1 = strlen( extra_ext1 );
	extra_len2 = strlen( extra_ext2 );
	extra_action = extra_ext2 + extra_len2 + 1;
	extra_len3 = strlen( extra_action );

	if ( !extra_len1 || !extra_len2 || !extra_len3 )
	{
	  	printf("\nNot a legal line: -X%s \n", extra_line);
		return;
	}

	strncpy(extra_extentions[ extra_entry ].target,
		extra_ext1,
		_MAX_EXT - 2 );
	strncpy(extra_extentions[ extra_entry ].source,
		extra_ext2,
		_MAX_EXT - 2 );
	strncpy(extra_extentions[ extra_entry ].command_line,
		extra_action,
		MAX_CMD_LINE - 1 );
}
/*--------------------------------------------------------------------------*/
unsigned short	options_make_all(	unsigned short	params,
					char		*param[] )
{
	hash_init( &g, GLOBAL_HASH_TABLE );
	hash_init( &h, LOCAL_HASH_TABLE );

	maker_open_makefile( file_args_num , file_args_vec );

	/* Fill all files in hash table                                     */
	maker_cur_dir( &h , file_args_num );

	maker_close_makefile( params, param );

	return MMAKER_OK;
}
/*--------------------------------------------------------------------------*/
void	options_usage( char *exe_name )
{
	printf( "\nUsage : %s {filename[s] | -{H|h|?}}\n",
#ifndef	MGEN_CASE_SENSITIVE_PATH
		lower_case_string( exe_name ) );
#else
		exe_name );
#endif
	printf("\n\tfilename   option is used to specify a profile for the MakeGENerator");
	printf("\n\t-{H|h|?}   option describes the parameters used in the profile");
	printf("\n\n\n\
Limitations:    Number of C source files           : %d\n\
		Number of recognized include files : %d\n",
		MAX_FILES,
		HASH_MAX_STATIC_ALLOCATION );
}
/*--------------------------------------------------------------------------*/
int	options_ext_match( char *name, char *ext )
{
	int	len_name = strlen( name );
	int	len_ext  = strlen( ext  );
	char	*eptr,
		*nptr;

	if ( len_ext >= ( len_name - 1 ) )
		return 1;

	eptr	= ext  + len_ext  - 1;
	nptr	= name + len_name - 1;

	while ( ( *eptr == *nptr ) && len_ext )
	{
		eptr--;
		nptr--;
		len_ext--;
	}
/*--------------------------------------------------------------------------*/

	if ( ( len_ext ) || ( *nptr != '.' ) )
		return 2;

	return 0;
}
/*--------------------------------------------------------------------------*/
unsigned short	options_is_this_a_pattern( char *str )
{
	char	*endp;


	if ( str == NULL )
		return 0;

	endp = str + strlen( str ) - 1;

	/*
	 * If there is no extention at all
	 */
	if ( *endp == '.' )
		return 0;

	while ( ( *endp != '.' ) && ( endp > str ) )
		endp--;

	if ( ( *endp == '.' ) && ( endp > str ) && ( *(endp - 1) == '*' ) )
		return 1;
	return 0;
}
/*--------------------------------------------------------------------------*/
unsigned short	options_get_path_of_str(char	*p,	/* source	*/
					char	*path )	/* destination	*/
{
	int	i = 0;
	int	len = strlen( p );
	while ( ( i < len ) && ( p[ i ] != '*' ) )
	{
		path[ i     ] = p[ i ];
		path[ i + 1 ] = '\0';
		i++;
	}
	if ( i == 0 )
	{
		path[ 0 ] = '\0';
	}
	else
	{
		if ( path[ i - 1 ] == PATH_SEPERATOR_CHAR )
			path[ i - 1 ] = '\0'; 

	}
	return 0;
}
/*--------------------------------------------------------------------------*/
unsigned short	options_get_extention_of_str(	char	*p,	/* source   */
						char	*ext )	/* dest     */
{
	char	*ptr = strstr( p, "*." );

	if ( ptr == NULL )
		ext[ 0 ] = '\0';
	else
		strcpy( ext, ptr + strlen( "*." ) );

	return 0;
}
/*--------------------------------------------------------------------------*/
#define OPTIONS_REPORT_COLLECT_OVERFLOW()		\
	fprintf(stderr,					\
		"\n%s\n%s %u %s\n%s",			\
		"Source file table is full.",		\
		"Can not handle more than",		\
		MAX_FILES,				\
		"files!",				\
		"The rest of the files will be ignored!"\
		);
/*--------------------------------------------------------------------------*/
unsigned short	options_can_strip_a_path(	char	*filename,
						char	*path,
						char	*name )
{
	char	*ptr;
	int	len;


	if (	( filename == NULL )	||
		( path     == NULL )	||
		( name     == NULL )	)
	{
		fprintf(stderr,
			"\nBad parameters sent to routine %s",
			"options_can_strip_a_path" );
		return 0;
	}
	ptr = filename + strlen( filename ) - 1;
	while (	( ptr > filename )			&&
		( ptr[ -1 ] != PATH_SEPERATOR_CHAR )	)
		ptr--;

	strcpy( name, ptr );
	len = strlen( filename ) - strlen( name );
	if ( len > 0 )
	{
		strncpy( path, filename, len );
		path[ len ] = '\0';
		if ( path[ strlen( path ) - 1 ] == PATH_SEPERATOR_CHAR )
			path[ strlen( path ) - 1 ] = '\0';
	}
	else
		path[ 0 ] = '\0';
	return 1;
}
/*--------------------------------------------------------------------------*/
unsigned short	options_collect_sources( void )
{
	DIR		*dirp;
	struct dirent	*dp;
	char		*p;
	char		*end_of_p;
	char		selection_copy[ MAX_FILES_SELECTION ];
	char		path[ _MAX_PATH ],
			name[ _MAX_PATH ],
			ext [ _MAX_EXT  ];
	char		*ret_path;

	file_args_vec_init();
#ifndef	MGEN_CASE_SENSITIVE_PATH
	lower_case_string( files_selection );
#endif
	strcpy( selection_copy, files_selection );

	p = strtok( selection_copy, ";" );
	while ( p != NULL )
	{
		/*
		 * If the next is not a pattern, but, a filename, no pattern
		 * matching will be used to select files.
		 */
		if ( !options_is_this_a_pattern( p ) )
		{
			if ( options_can_strip_a_path( p, path, name ) )
			{
				ret_path = file_args_path_insert( path );
				file_args_vec_insert( name, ret_path );
				p = strtok( NULL, ";" );
				continue;
			}
			if ( strlen( p ) >= ( MAX_FILE_NAME - 1 ) )
			{
				fprintf(stderr,
					"\nFilename too long: %s. Ignored.",
					p );
				p = strtok( NULL, ";" );
				continue;
			}
			file_args_vec_insert( p, file_args_path );
			p = strtok( NULL, ";" );
			continue;
		}
		/*
		 * The next is a pattern of the form "*.extention",
		 *   or, "[path]PATH_SEPERATOR_CHAR*.extention".
		 */
		options_get_path_of_str( p, path );
		options_get_extention_of_str( p, ext );
		end_of_p = p + strlen( p ) - 1;
		while ( *end_of_p != '.' )
			end_of_p--;
		end_of_p++;

		if ( path[ 0 ] == '\0' )
			dirp = opendir( "." );
		else
			dirp = opendir( path );
		if ( dirp == NULL )
		{
			fprintf(stderr,
				"\nCannot open \"%s\" directory",
				path );
			p = strtok( NULL, ";" );
			continue;
		}
		ret_path = file_args_path_insert( path );

		while ( (dp = readdir( dirp )) != NULL )
		{
#ifndef	MGEN_CASE_SENSITIVE_PATH
			if ( options_ext_match( lower_case_string( dp->d_name ),
#else
			if ( options_ext_match( dp->d_name,
#endif
							    end_of_p ) == 0 )
			{
				if ( p[ 0 ] != '*' )
				{
					file_args_vec_insert(	dp->d_name,
								ret_path );
				}
				else
				{
					file_args_vec_insert(	dp->d_name,
								file_args_path);
				}
				if (file_args_num >= MAX_FILES)
				{
					OPTIONS_REPORT_COLLECT_OVERFLOW();
					closedir( dirp );
					return	0;
				}
			}
			/*
			 * Get the next file from currently
			 * open directory.
			 */
		}
		closedir( dirp );
		p = strtok( NULL, ";" );
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
void	options_compute_base_directory()
{
	unsigned short	len;
	unsigned short	i;
	char		last_char;

	getcwd( base_working_directory , _MAX_PATH );
	len = strlen( base_working_directory );

	for ( i = len - 1 ; i > 0 ; i-- )
	{
		last_char = base_working_directory[ i ];
		base_working_directory[ i ] = '\0';
		if ( last_char == PATH_SEPERATOR_CHAR )
			break;
	}
#ifndef	MGEN_CASE_SENSITIVE_PATH
	lower_case_string( base_working_directory );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_full_target_name()
{
	unsigned short	len;
	char		*cptr;

	if ( options_target_name[ 0 ] == '\0' )
	{
		len = strlen( base_working_directory );
		cptr = base_working_directory + len - 1;

		while ( cptr[ -1 ] != PATH_SEPERATOR_CHAR )
			cptr--;
	}
	else
		cptr = options_target_name;


	if ( options_spc_target_dir[ 0 ] == '\0' )
		sprintf(full_target_name,
			((options_spc_extention[0]) ? "%s%c%s.%s" : "%s%c%s" ),
			base_wd_string,
			PATH_SEPERATOR_CHAR,
			cptr,
			options_spc_extention);
	else
		sprintf(full_target_name,
			((options_spc_extention[0]) ? "%s%c%s%c%s.%s" : "%s%c%s%c%s" ),
			base_wd_string,
			PATH_SEPERATOR_CHAR,
			options_spc_target_dir,
			PATH_SEPERATOR_CHAR,
			cptr,
			options_spc_extention);

#ifndef	MGEN_CASE_SENSITIVE_PATH
	if (maker_control_bwd != True)
		lower_case_string( full_target_name );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_linker_output_dir_name()
{
	if ( options_spc_target_dir[ 0 ] == '\0' )
		sprintf(linker_output_directory,
			"%s",
			base_wd_string );
	else
		sprintf(linker_output_directory,
			"%s%c%s",
			base_wd_string,
			PATH_SEPERATOR_CHAR,
			options_spc_target_dir );

#ifndef	MGEN_CASE_SENSITIVE_PATH
	if (maker_control_bwd != True)
		lower_case_string( linker_output_directory );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_linker_run_dir_name()
{
	if ( options_spc_run_linker_dir[ 0 ] == '\0' )
		sprintf(linker_run_directory,
			"%s",
			base_wd_string );
	else
		sprintf(linker_run_directory,
			"%s%c%s",
			base_wd_string,
			PATH_SEPERATOR_CHAR,
			options_spc_run_linker_dir );

#ifndef	MGEN_CASE_SENSITIVE_PATH
	if (maker_control_bwd != True)
		lower_case_string( linker_run_directory );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_compiler_output_dir_name()
{
	if ( options_spc_obj_dir[ 0 ] == '\0' )
		sprintf(compiler_output_directory,
			"%s",
			base_wd_string );
	else
		sprintf(compiler_output_directory,
			"%s%c%s",
			base_wd_string,
			PATH_SEPERATOR_CHAR,
			options_spc_obj_dir );

#ifndef	MGEN_CASE_SENSITIVE_PATH
	if (maker_control_bwd != True)
		lower_case_string( compiler_output_directory );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_compiler_run_dir_name()
{
	unsigned short	len;
	char		buffer[ _MAX_PATH ];


	getcwd( buffer, _MAX_PATH );
#ifndef	MGEN_CASE_SENSITIVE_PATH
	lower_case_string( buffer );
#endif
	len = strlen( buffer ) - 1;
   
	while ( buffer[ len ] != PATH_SEPERATOR_CHAR )
		len--;
	len++;
	sprintf(compiler_run_directory,
		"%s%c%s",
		base_wd_string,
		PATH_SEPERATOR_CHAR,
		( buffer + len ) );

#ifndef	MGEN_CASE_SENSITIVE_PATH
	if (maker_control_bwd != True)
		lower_case_string( compiler_run_directory );
#endif
}
/*--------------------------------------------------------------------------*/
void	options_compute_general_variables()
{
  	options_compute_base_directory();
	if (maker_base_wd == BASE_WD_IS_FULL)
		strcpy( BASE_WD, base_working_directory );
	strcpy( base_wd_string, ((maker_control_bwd == True) ? "$(BASE_WD)" : base_working_directory) );
	options_compute_full_target_name();
	options_compute_linker_output_dir_name();
	options_compute_linker_run_dir_name();
	options_compute_compiler_output_dir_name();
	options_compute_compiler_run_dir_name();
}
/*--------------------------------------------------------------------------*/
char	*options_indent_1tab( char *cptr )
{
  	unsigned short	i;

	for ( i = 0; (cptr[ i ] && cptr[ i + 1 ]) != 0; i++ )
		if (( cptr[ i ] == '\\' ) && ( cptr[ i + 1 ] == 'n' ))
		{
			cptr[ i ]	= '\n';
			cptr[ i + 1 ]	= '\t';
			while (isspace( cptr[ i + 2 ] ))
			{
				cptr[ i ]	= ' ';
				cptr[ i + 1 ]	= '\n';
				cptr[ i + 2 ]	= '\t';
				i++;
			}
			i++;
		}
	return	cptr;
}
/*--------------------------------------------------------------------------*/
char	*options_undent_1tab( char *cptr )
{
  	unsigned short	i;
	char		a_copy[ MAX_PROFILE_LINE_LENGTH ];
	unsigned short	len;
	char		*tmp = cptr;

	strncpy( a_copy, cptr, MAX_PROFILE_LINE_LENGTH );
        a_copy[ MAX_PROFILE_LINE_LENGTH - 1 ] = '\0';
	len = strlen( a_copy );
	for ( i = 0; i < len; i++ )
		if (( a_copy[ i ] == '\n' ) && ( a_copy[ i + 1 ] == '\t' ))
		{
		  	*tmp++ = '\\';
		  	*tmp++ = 'n';
			i++;
		}
		else
			*tmp++ = a_copy[ i ];

	return	cptr;
}
/*--------------------------------------------------------------------------*/
void	options_save_linker_separate_lines(	FILE	*f_ptr,
						char	*str1,
						char	*str2,
						char	first_char,
						char	other_chars,
						char	letter	)
{
	char	*ptr;
	int	first = 1;
	int	i;

	while ( strlen( str1 ) )
	{
		ptr = strstr(str1, str2);
		if ( ptr != NULL )
			*ptr = '\0';

		fprintf( f_ptr, "\n%c%c%s",
				(first ? first_char : other_chars),
				letter,
				str1 );
		if ( first )
			first = 0;
		if ( ptr == NULL )
			break;
		for ( i = 0; i < strlen( str2 ); i++ )
			ptr[ i ] = str2[ i ];
		if ( strlen( ptr + strlen( str2 ) ) )
			str1 = ptr + strlen( str2 );
		else
			break;
	}
}
/*--------------------------------------------------------------------------*/
unsigned short 	options_read_profile(	unsigned short	params,
					char		*param[]	)
{
	unsigned short	i	= 1;
	unsigned short	ret_val	= 0;


	options_init_extra_extentions();
	if ( params < 2 )
	{
		options_usage( param[ 0 ] );
		return	NOT_ENOUGH_PARAMS;
	}
	xxxx_macros_init();

	while ( i < params )
	{
		if ( param[ i ][ 0 ] != '-' )
			ret_val |= options_load_setup( param[ i ] );
		else
			if (	( param[ i ][ 1 ] != 'H')	&&
		  		( param[ i ][ 1 ] != 'h')	&&
		  		( param[ i ][ 1 ] != '?')	)
			{
				options_usage( param[ 0 ] );
				return	NO_PROFILE_PARAM;
			}
			else
			{
				/* 					   */
				/* Show help about MakeGENerator	   */
				/* 					   */
				options_help();
				return	NO_PROFILE_PARAM;
			}
		i++;
	}
	xxxx_macros_merge();

	xxxx_pathes_init();
	xxxx_pathes_read_path_str( path_str );
	xxxx_pathes_merge();
	xxxx_pathes_write_path_str( path_str );

	return ret_val;
}
/*--------------------------------------------------------------------------*/
#define ADVANCE		cptr++;					\
			while (isspace( cptr[ 0 ] ))		\
				cptr++;
#define IF_NOT_EMPTY	if ( cptr[ 0 ] != '\0' )
/*--------------------------------------------------------------------------*/
unsigned short	options_load_setup( char	*cptr )
{
	FILE		*profile;
	char		prof_line[ MAX_PROFILE_LINE_LENGTH ];
	unsigned short 	len	= 0;
	unsigned short 	path_len= 0;
	unsigned short 	tmp_i;


	profile = fopen( cptr, "rt" );
	if ( profile == NULL )
	{
		printf( "\nCannot open profile named %s!", cptr );
		return	BAD_PROFILE_PARAM;
	}

	while ( fgets( prof_line, MAX_PROFILE_LINE_LENGTH - 1, profile ) != NULL )
	{
		cptr = prof_line;
		while ( isspace( cptr[ 0 ] ) )
			cptr++;
		len = strlen( cptr );
		while ( ( len != 0 )		&&
			( isspace( cptr[ len - 1 ] ) ) )
			cptr[ --len ] = '\0';
		if (( cptr[ 0 ] != '-' )	&&
		    ( cptr[ 0 ] != '+' )	)
			continue;

		if ( cptr[ 0 ] == '+' )
		{
			cptr++;
			len--;

			switch ( cptr[ 0 ] )
			{
			case 'L':
				ADVANCE		
				IF_NOT_EMPTY
				{
					int	tmp;

					cptr = options_indent_1tab( cptr );
					tmp = strlen( cptr ) + strlen( LINKER ) + 2;
					if ( (MAX_LIB_CMD_LINE -1 ) > tmp )
					{
						strcat( LINKER, "\n\t" );
						strcat( LINKER, cptr );
					}
				}
				break;
			case 'C':
				ADVANCE		
				IF_NOT_EMPTY
				{
					int	tmp;

					tmp = strlen( cptr ) + strlen( COMPILER ) + 2;
					if ( (MAX_LIB_CMD_LINE -1) > tmp )
					{
						strcat( COMPILER, "\\n" );
						strcat( COMPILER, cptr );
					}
				}
				break;
			case 'm':
				xxxx_macros_handle_line( cptr - 1 );
				break;
			case 'P':
				ADVANCE		
				IF_NOT_EMPTY
				{
					int	tmp;

					tmp = strlen( cptr ) + strlen( path_str );
					if ( (PATH_MAX_LEN -1 ) > tmp )
					{
						strcat( path_str, cptr );
					}
					path_len = strlen( path_str );
					while ( isspace( path_str[ path_len - 1 ] ) )
						path_str[ --path_len ] = '\0';
					if ( (PATH_MAX_LEN -1 ) > (strlen( path_str ) + 1) )
					{
						strcat( path_str, ";" );
					}
				}
				break;

			case 'D':
				ADVANCE		
				IF_NOT_EMPTY
				{
				  	if ( linker_dependencies[ strlen( linker_dependencies ) - 1 ] != ';' )
						strcat( linker_dependencies, ";" );
					if ( ( strlen( linker_dependencies ) + strlen( cptr ) ) < MAX_PROFILE_LINE_LENGTH )
						strcat( linker_dependencies, cptr );
				}
				break;

			case 'd':
				ADVANCE		
				IF_NOT_EMPTY
				{
				  	if ( compiler_dependencies[ strlen( compiler_dependencies ) - 1 ] != ';' )
						strcat( compiler_dependencies, ";" );
					if ( ( strlen( compiler_dependencies ) + strlen( cptr ) ) < MAX_PROFILE_LINE_LENGTH )
						strcat( compiler_dependencies, cptr );
				}
				break;
		case 'E':
			ADVANCE		
			if (	( strlen( files_selection ) + strlen( cptr ) ) <
				( MAX_FILES_SELECTION - 2 ) )
			{
				strcat( files_selection, ";" );
				strcat( files_selection, cptr );
#ifndef	MGEN_CASE_SENSITIVE_PATH
				lower_case_string( files_selection );
#endif
			}
			break;

			default:
				fclose( profile );
				printf(	"\nERROR: flag +%s is not supported by this version!",
					cptr );
				return	MMAKER_NOT_OK;
			}
			continue;
		}

		cptr++;
		len--;

		switch ( cptr[ 0 ] )
		{
		case 'm':
			xxxx_macros_handle_line( cptr - 1 );
			break;
		case 'P':
			ADVANCE		
			IF_NOT_EMPTY
			{
				strncpy( path_str, cptr, PATH_MAX_LEN - 1 );
				path_len = strlen( path_str );
				while ( isspace( path_str[ path_len - 1 ] ) )
					path_str[ --path_len ] = '\0';
				if ( (PATH_MAX_LEN -1 ) > (strlen( path_str ) + 1) )
				{
					strcat( path_str, ";" );
				}
			}
			break;
		case 'O':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_spc_target_dir, cptr, _MAX_PATH - 1 );
			break;

		case 'R':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_spc_run_linker_dir, cptr, _MAX_PATH - 1 );
			break;

		case 'o':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_spc_obj_dir, cptr, _MAX_PATH );
			break;

		case 'L':
			ADVANCE		
			IF_NOT_EMPTY
			{
				cptr = options_indent_1tab( cptr );
				strncpy( LINKER, cptr, MAX_LIB_CMD_LINE - 1 );
			}
			break;
		case 'l':
			ADVANCE		
			IF_NOT_EMPTY
			  	maker_max_lines = (unsigned short)atoi( cptr );
			break;
		case 'C':
			ADVANCE		
			IF_NOT_EMPTY
			{
				strncpy( COMPILER, cptr, MAX_CMD_LINE - 1 );
			}
			break;
		case 'T':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_spc_extention, cptr, 3 );
			break;
		case 't':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_target_name, cptr, _MAX_PATH - 2 );
			break;
		case 'F':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( LINKER_FLAGS, cptr, _MAX_PATH );
			break;
		case 'f':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( COMPILER_FLAGS, cptr, MAX_CMD_LINE - 1 );
			break;
		case 'M':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( MAKEFILE, cptr, _MAX_PATH - 1 );
			break;
		case 'D':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( linker_dependencies, cptr, MAX_PROFILE_LINE_LENGTH - 1 );
			break;
		case 'd':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( compiler_dependencies, cptr, MAX_PROFILE_LINE_LENGTH - 1 );
			break;
		case 'E':
			ADVANCE		
			if ( strlen( cptr ) >= 3 )
			{
				strncpy( files_selection, cptr, MAX_FILES_SELECTION - 1 );
#ifndef	MGEN_CASE_SENSITIVE_PATH
				lower_case_string( files_selection );
#endif
			}
			break;
		case 'e':
			ADVANCE		
			if ( strlen( cptr ) )
				strncpy( output_files_extention, cptr, _MAX_EXT - 2 );
			for ( tmp_i = 1; tmp_i < ( _MAX_EXT - 2 ); tmp_i++ )
				if ( isspace( output_files_extention[ tmp_i ] ) )
					output_files_extention[ tmp_i ] = '\0';
			break;
		case 'B':
			ADVANCE		
			cptr++;
			if ( strcmp( cptr, "full" ) == 0 )
				maker_base_wd = BASE_WD_IS_FULL;
			break;
		case 'N':
			ADVANCE		
			if ( strcmp( cptr, "bwd" ) == 0 )
				maker_control_bwd = True;
			if ( strcmp( cptr, "bwd+" ) == 0 )
				maker_control_bwd = True;
			if ( strcmp( cptr, "bwd-" ) == 0 )
				maker_control_bwd = False;

			if ( strcmp( cptr, "exe" ) == 0 )
				maker_control_exe = True;
			if ( strcmp( cptr, "exe+" ) == 0 )
				maker_control_exe = True;
			if ( strcmp( cptr, "exe-" ) == 0 )
				maker_control_exe = False;

			if ( strcmp( cptr, "obj" ) == 0 )
				maker_control_obj = True;
			if ( strcmp( cptr, "obj+" ) == 0 )
				maker_control_obj = True;
			if ( strcmp( cptr, "obj-" ) == 0 )
				maker_control_obj = False;

			if ( strcmp( cptr, "inc" ) == 0 )
				maker_control_inc = True;
			if ( strcmp( cptr, "inc+" ) == 0 )
				maker_control_inc = True;
			if ( strcmp( cptr, "inc-" ) == 0 )
				maker_control_inc = False;

			if ( strncmp( cptr, "inc_prefix", strlen("inc_prefix") ) == 0 )
			{
				cptr = cptr + strlen("inc_prefix");
				if ( cptr[ 0 ] != '\0' )
					cptr++;
				strncpy( options_inc_prefix, cptr, 9 );
			}

			if ( strcmp( cptr, "com" ) == 0 )
				maker_control_com = True;
			if ( strcmp( cptr, "com+" ) == 0 )
				maker_control_com = True;
			if ( strcmp( cptr, "com-" ) == 0 )
				maker_control_com = False;

			if ( strcmp( cptr, "lin" ) == 0 )
				maker_control_lin = True;
			if ( strcmp( cptr, "lin+" ) == 0 )
				maker_control_lin = True;
			if ( strcmp( cptr, "lin-" ) == 0 )
				maker_control_lin = False;

			if ( strcmp( cptr, "rel" ) == 0 )
				maker_control_rel = True;
			if ( strcmp( cptr, "rel+" ) == 0 )
				maker_control_rel = True;
			if ( strcmp( cptr, "rel-" ) == 0 )
				maker_control_rel = False;

			if ( strcmp( cptr, "mis" ) == 0 )
				maker_control_mis = True;
			if ( strcmp( cptr, "mis+" ) == 0 )
				maker_control_mis = True;
			if ( strcmp( cptr, "mis-" ) == 0 )
				maker_control_mis = False;

			if ( strcmp( cptr, "source_files" ) == 0 )
				maker_control_source_files = True;
			if ( strcmp( cptr, "source_files+" ) == 0 )
				maker_control_source_files = True;
			if ( strcmp( cptr, "source_files-" ) == 0 )
				maker_control_source_files = False;

			if ( strcmp( cptr, "make_depend" ) == 0 )
				maker_control_make_depend = True;
			if ( strcmp( cptr, "make_depend+" ) == 0 )
				maker_control_make_depend = True;
			if ( strcmp( cptr, "make_depend-" ) == 0 )
				maker_control_make_depend = False;

			if ( strcmp( cptr, "flat_P" ) == 0 )
				maker_control_flat_path_str = True;
			if ( strcmp( cptr, "flat_P+" ) == 0 )
				maker_control_flat_path_str = True;
			if ( strcmp( cptr, "flat_P-" ) == 0 )
				maker_control_flat_path_str = False;

			if ( strcmp( cptr, "flat_L" ) == 0 )
				maker_control_flat_linker = True;
			if ( strcmp( cptr, "flat_L+" ) == 0 )
				maker_control_flat_linker = True;
			if ( strcmp( cptr, "flat_L-" ) == 0 )
				maker_control_flat_linker = False;

			if ( strcmp( cptr, "flat_C" ) == 0 )
				maker_control_flat_compiler = True;
			if ( strcmp( cptr, "flat_C+" ) == 0 )
				maker_control_flat_compiler = True;
			if ( strcmp( cptr, "flat_C-" ) == 0 )
				maker_control_flat_compiler = False;

			if ( strcmp( cptr, "flat_D" ) == 0 )
				maker_control_flat_Dependencies = True;
			if ( strcmp( cptr, "flat_D+" ) == 0 )
				maker_control_flat_Dependencies = True;
			if ( strcmp( cptr, "flat_D-" ) == 0 )
				maker_control_flat_Dependencies = False;

			if ( strcmp( cptr, "flat_d" ) == 0 )
				maker_control_flat_dependencies = True;
			if ( strcmp( cptr, "flat_d+" ) == 0 )
				maker_control_flat_dependencies = True;
			if ( strcmp( cptr, "flat_d-" ) == 0 )
				maker_control_flat_dependencies = False;
				
			/*
			 * If a "-Nflat_inc?" switch is encounterd, we have
			 * to activate the "-Ninc" flag, so that it will
			 * have an affect.
			 */
			if ( strcmp( cptr, "flat_inc" ) == 0 )
			{
				maker_control_flat_inc = True;
				maker_control_inc = True;
			}
			if ( strcmp( cptr, "flat_inc+" ) == 0 )
			{
				maker_control_flat_inc = True;
				maker_control_inc = True;
			}
			if ( strcmp( cptr, "flat_inc-" ) == 0 )
			{
				maker_control_flat_inc = False;
				maker_control_inc = True;
			}

			if ( strcmp( cptr, "flat_E" ) == 0 )
				maker_control_flat_files_selection = True;
			if ( strcmp( cptr, "flat_E+" ) == 0 )
				maker_control_flat_files_selection = True;
			if ( strcmp( cptr, "flat_E-" ) == 0 )
				maker_control_flat_files_selection = False;
				
			if ( strcmp( cptr, "use_logical_pathes" ) == 0 )
				maker_control_use_logical_pathes = True;
			if ( strcmp( cptr, "use_logical_pathes+" ) == 0 )
				maker_control_use_logical_pathes = True;
			if ( strcmp( cptr, "use_logical_pathes-" ) == 0 )
				maker_control_use_logical_pathes = False;
				
			break;
		case 'X':
			ADVANCE		
			IF_NOT_EMPTY
				options_interpret_extra_extentions( cptr );
			break;
		case 'I':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_user_header_filename, cptr, _MAX_PATH - 1 );
			break;
		case 'i':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_user_footer_filename, cptr, _MAX_PATH - 1 );
			break;
		case 'S':
			ADVANCE		
			IF_NOT_EMPTY
			{
				if ( strcmp( cptr, "spc" ) == 0 )
					strcpy( dependency_seperator, TWO_SPACES_DEPENDENCY_SEPERATOR );
				else	/* Could be "nl"		    */
					strcpy( dependency_seperator, NEWLINE_TAB_TAB_DEPENDENCY_SEPERATOR );
			}
			break;
		case 's':
			ADVANCE		
			IF_NOT_EMPTY
				strncpy( options_save_setup_filename, cptr, _MAX_PATH - 1 );
			break;

		default:
			fclose( profile );
			printf(	"\nERROR: flag -%s is not supported by this version!",
				cptr );
			return	MMAKER_NOT_OK;
		}
	}

	fclose( profile );
	return MMAKER_OK;
}
/*--------------------------------------------------------------------------*/
unsigned short	options_save_setup( char	*cptr )
{
	FILE		*profile;
	char		prof_line[ MAX_PROFILE_LINE_LENGTH ];
	unsigned short	i;
#ifdef	BORLANDC_DOS_VERSION
	struct dosdate_t	d;
	struct dostime_t	t;
#endif
#ifdef	BORLANDC_WINDOWS_VERSION
	struct dosdate_t	d;
	struct dostime_t	t;
#endif
#ifdef	HIGHC_DOS_VERSION
	struct _dosdate_t	d;
	struct _dostime_t	t;
#endif
#ifdef	GNUC_LINUX_VERSION
	time_t			tt;
	struct tm		*tmm;
#endif

	xxxx_pathes_write_original_path_str( path_str );

	profile = fopen( cptr, "wt" );
	if ( profile == NULL )
	{
		printf( "\nCannot open profile named %s for writing!", cptr );
		return	BAD_PROFILE_PARAM;
	}

	fprintf( profile , "#");
	fprintf( profile , "\n#");
	fprintf(profile,
		"\n# Gil-ad Madar's MakeGENerator.\t\t\t\t\t version %s%c",	\
		version_number,							\
		MAKEGEN_VER_CHAR );

#ifndef GNUC_LINUX_VERSION
	_dos_getdate(&d);
	_dos_gettime(&t);
#else
	time( &tt );
	tmm = localtime( &tt );
#endif

	fprintf(profile,
		"\n# DATE: %02d:%02d:%04d\t\t\t\t\t\tTIME: %02d:%02d:%02d",
#ifndef GNUC_LINUX_VERSION
		d.day,
		d.month,
		d.year,
		t.hour,
		t.minute,
		t.second );
#else
		tmm->tm_mday,
		tmm->tm_mon,
		tmm->tm_year + 1900,
		tmm->tm_hour,
		tmm->tm_min,
		tmm->tm_sec );
#endif

	fprintf( profile , "\n#");
	fprintf( profile , "\n#");
	fprintf( profile , "\n# This profile was automatically generated for MakeGENerator.");
	fprintf( profile , "\n#");

	xxxx_macros_print( profile );

	if ( path_str[ 0 ] != '\0' )
	{
		strcpy( prof_line, path_str );
		if ( prof_line[ strlen( prof_line ) - 1 ] == ';' )
			prof_line[ strlen( prof_line ) - 1 ] = '\0';
		if ( maker_control_flat_path_str == True )
			fprintf(profile,
				"\n-%c%s",
				'P',
		 		prof_line );
		else
			options_save_linker_separate_lines(	profile,
								prof_line,
								";",
								'-', '+', 'P' );
	}

	if ( options_spc_target_dir[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'O',
			options_spc_target_dir );

	if ( options_spc_run_linker_dir[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'R',
			options_spc_run_linker_dir );

	if ( options_spc_obj_dir[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'o',
			options_spc_obj_dir );

	strcpy( prof_line, LINKER );
	options_undent_1tab( prof_line );
	if ( prof_line[ 0 ] != '\0' )
	{
		if ( maker_control_flat_linker == True )
			fprintf(profile,
				"\n-%c%s",
				'L',
		 		prof_line );
		else
			options_save_linker_separate_lines(	profile,
								prof_line,
								"\\n",
								'-', '+', 'L' );
	}

	fprintf(profile,
		"\n-%c%u",
		'l',
		maker_max_lines );

	strcpy( prof_line, COMPILER );
	if ( prof_line[ 0 ] != '\0' )
	{
		if ( maker_control_flat_compiler == True )
		{
			if ( prof_line[ 0 ] != '\0' )
				fprintf(profile,
					"\n-%c%s",
					'C',
					prof_line );
		}
		else
			options_save_linker_separate_lines(	profile,
								prof_line,
								"\\n",
								'-', '+', 'C' );
	}


	if ( options_spc_extention[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'T',
			options_spc_extention );


	if ( options_target_name[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			't',
		 	options_target_name );

	if ( LINKER_FLAGS[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'F',
		 	LINKER_FLAGS );

	if ( COMPILER_FLAGS[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'f',
		 	COMPILER_FLAGS );

	fprintf(profile,
		"\n-%c%s",
		'M',
		 MAKEFILE );

	if ( linker_dependencies[ 0 ] != '\0' )
	{
		strcpy( prof_line, linker_dependencies );
		if ( prof_line[ strlen( prof_line ) - 1 ] == ';' )
			prof_line[ strlen( prof_line ) - 1 ] = '\0';
	  	if ( maker_control_flat_Dependencies == True )
			fprintf(profile,
				"\n-%c%s",
				'D',
			 	prof_line );
		else
			options_save_linker_separate_lines(	profile,
								prof_line,
								";",
								'-', '+', 'D' );
	}

	if ( compiler_dependencies[ 0 ] != '\0' )
	{
		strcpy( prof_line, compiler_dependencies );
		if ( prof_line[ strlen( prof_line ) - 1 ] == ';' )
			prof_line[ strlen( prof_line ) - 1 ] = '\0';
	  	if ( maker_control_flat_dependencies == True )
			fprintf(profile,
				"\n-%c%s",
				'd',
			 	prof_line );
	  	else
			options_save_linker_separate_lines(	profile,
								prof_line,
								";",
								'-', '+', 'd' );
	}


	strcpy( prof_line, files_selection );
	if ( prof_line[ strlen( prof_line ) - 1 ] == ';' )
		prof_line[ strlen( prof_line ) - 1 ] = '\0';
	if ( maker_control_flat_files_selection == True )
		fprintf(profile,
			"\n-%c%s",
			'E',
			files_selection );
	else
			options_save_linker_separate_lines(	profile,
								prof_line,
								";",
								'-', '+', 'E' );

	fprintf(profile,
		"\n-%c%s",
		'e',
		output_files_extention );

	switch ( maker_base_wd )
	{
	case BASE_WD_IS_UWD:
		strcpy( prof_line, "uwd" );
		break;
	case BASE_WD_IS_FULL:
		strcpy( prof_line, "full" );
		break;
	}
	fprintf(profile,
		"\n-%c%s",
		'B',
		 prof_line );

	if ( maker_control_bwd == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
		 	"bwd" );
	if ( maker_control_exe == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"exe" );
	if ( maker_control_obj == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"obj" );
	if ( maker_control_inc == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"inc" );
	if ( options_inc_prefix[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s %s",
			'N',		  
			"inc_prefix",
			options_inc_prefix );
	if ( maker_control_com == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"com" );
	if ( maker_control_lin == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"lin" );
	if ( maker_control_rel == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"rel" );
	if ( maker_control_mis == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"mis" );
	if ( maker_control_source_files == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"source_files" );
	if ( maker_control_make_depend == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"make_depend" );
	if ( maker_control_flat_path_str == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_P" );
	if ( maker_control_flat_linker == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_L" );
	if ( maker_control_flat_compiler == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_C" );
	if ( maker_control_flat_Dependencies == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_D" );
	if ( maker_control_flat_dependencies == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_d" );
	if ( maker_control_flat_inc == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_inc" );
	if ( maker_control_flat_files_selection == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"flat_E" );
	if ( maker_control_use_logical_pathes == True )
		fprintf(profile,
			"\n-%c%s",
			'N',
			"use_logical_pathes" );
	

	for ( i = 0; i < 10; i++ )
	{
	  	if (	(extra_extentions[ i ].target[ 0 ] != '\0')	&&
			(extra_extentions[ i ].source[ 0 ] != '\0')	&&
			(extra_extentions[ i ].command_line[ 0 ] != '\0')	)
			fprintf(profile,
				"\n-X%d%s.%s.%s",
				i,
				extra_extentions[ i ].target,
				extra_extentions[ i ].source,
				extra_extentions[ i ].command_line	);
	}

	if ( options_user_header_filename[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'I',
			options_user_header_filename );
	if ( options_user_footer_filename[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			'i',
			options_user_footer_filename );

	fprintf(profile,
		"\n-%c%s",
		'S',
		(( strcmp( dependency_seperator, "spc" ) == 0 ) ? "spc" : "nl" )
		);

	if ( options_save_setup_filename[ 0 ] != '\0' )
		fprintf(profile,
			"\n-%c%s",
			's',
			options_save_setup_filename
			);

	fclose( profile );
	return MMAKER_OK;
}
/*--------------------------------------------------------------------------*/
#define HELP_PUT_DASHED_LINE	\
	printf("\n#-----------------------------------------------------------------------------#");
/*--------------------------------------------------------------------------*/
void	options_help()
{
#ifdef	BORLANDC_DOS_VERSION
	textcolor( 15 );
	textbackground( 1 );
	clrscr();
#endif
	PUT_MAKEGEN_LOGO
	printf("\nThe following table describes the legal parameters that may");
	printf("\nappear in the profile given to the MakeGENerator:");
	HELP_PUT_DASHED_LINE
	printf("\n| FLAG MEANING                    KIND OF VALUES    VALUES         DEFAULT    |");
	HELP_PUT_DASHED_LINE
	printf("\n| -P   [INCLUDE_PATH]                  text       d:\\bc\\include    EMPTY      |");
	printf("\n| +P   [INCLUDE_PATH] - append         text       d:\\bc\\include    EMPTY      |");
	printf("\n| -R   linker's run directory          text           linkdir      src        |");
	printf("\n| -O   linker's output directory       text           libdir       EMPTY      |");
	printf("\n| -o   compiler's output directory     text            ext         EMPTY      |");
	printf("\n| -L   [LINKER]                        text            text        lib        |");
	printf("\n| +L   [LINKER] - append               text            text        EMPTY      |");
	printf("\n| -l   max. search lines per file     integer          10          0 (all)    |");
	printf("\n| -C   [COMPILER]                      text            text        ce         |");
	printf("\n| +C   [COMPILER] - append             text            text        EMPTY      |");
	printf("\n| -M   makefile's name                 text            text        makefile   |");
	printf("\n| -D   linker's specific depnd. list   text         target.lnk     EMPTY      |");
	printf("\n| +D   linker's depnd. list - append   text         target.lnk     EMPTY      |");
	printf("\n| -d   compiler's specific depnd. list text         turboc.cfg     EMPTY      |");
	printf("\n| +d   compiler's depnd. list - append text         turboc.cfg     EMPTY      |");
	printf("\n| -T   target type                     text            exp         lib        |");
	printf("\n| -t   target name                     text           myapp        EMPTY      |");
	printf("\n| -e   output files extention          text            text        obj        |");
#ifndef	BORLANDC_WINDOWS_VERSION
	HELP_PUT_DASHED_LINE
	getchar();
#endif
#ifdef	BORLANDC_DOS_VERSION
	clrscr();
#endif
#ifndef	BORLANDC_WINDOWS_VERSION
	HELP_PUT_DASHED_LINE
	printf("\n| FLAG MEANING                    KIND OF VALUES    VALUES         DEFAULT    |");
	HELP_PUT_DASHED_LINE
#endif
	printf("\n| -E   input files extention           text            *.c;*.cpp   *.c        |");
	printf("\n| +E   input files extention append    text            *.c;fil.cpp *.c        |");
	printf("\n| -F   [LINKER_FLAGS]                  text            text        EMPTY      |");
	printf("\n| -f   [COMPILER_FLAGS]                text            text        EMPTY      |");
	printf("\n| -B   [BASE_WD]                       specific        uwd         uwd        |");
	printf("\n|                                                      full                   |");
	printf("\n| -I   header file                     text          myheader      EMPTY      |");
	printf("\n| -i   footer file                     text          myfooter      EMPTY      |");
	printf("\n| -N   [BASE_WD]                       specific        bwd[+- ]    False      |");
	printf("\n|      [EXE_FILE]                                      exe[+- ]    False      |");
	printf("\n|      [OBJECTS]                                       obj[+- ]    False      |");
	printf("\n|      [INCLUDE_PATH]                                  inc[+- ]    False      |");
	printf("\n|      [COMPILER]                                      com[+- ]    False      |");
	printf("\n|      [LINKER]                                        lin[+- ]    False      |");
	printf("\n|      Use relative names for local files              rel[+- ]    False      |");
	printf("\n|      Specify the missing files                       mis[+- ]    False      |");
	printf("\n|      flat Path lines in profile                   flat_P[+- ]    False      |");
	printf("\n|      flat Link lines in profile                   flat_L[+- ]    False      |");
	printf("\n|      flat Compile lines in profile                flat_C[+- ]    False      |");
	printf("\n|      flat Dependency lines in profile             flat_D[+- ]    False      |");
	printf("\n|      flat dependency lines in profile             flat_d[+- ]    False      |");
	HELP_PUT_DASHED_LINE
	getchar();

#ifdef	BORLANDC_DOS_VERSION
	clrscr();
#endif
#ifndef	BORLANDC_WINDOWS_VERSION
	HELP_PUT_DASHED_LINE
	printf("\n| FLAG MEANING                    KIND OF VALUES    VALUES         DEFAULT    |");
	HELP_PUT_DASHED_LINE
	printf("\n| -N   [SOURCE_FILES]                  specific source_files[+- ]  False      |");
	printf("\n|      generate a depend: node         specific  make_depend[+- ]  False      |");
	printf("\n|      flat INCLUDE_PATH def.          specific   flat_inc[+- ]    False      |");
	printf("\n|      flat files_selection def.       specific     flat_E[+- ]    False      |");
	printf("\n|      include path prefix       \"inc_prefix \"text     text        EMPTY      |");
	printf("\n|      use macros in pathes     specific  use_logical_pathes[+- ]  False      |");
	printf("\n| -m   start macro definition   -mINC1 = c:\\bc\\include text        EMPTY      |");
	printf("\n| +m   start macro definition   -mINC2 = c:\\c\\include  text        EMPTY      |");
	printf("\n| -X   extra extentions          [0-9]trg.src.tool obj.asm.asm386  EMPTY      |");
	printf("\n| -S   seperator type                  specific        spc         nl         |");
	printf("\n|                                                      nl                     |");
	printf("\n| -s   save setup filename             text            text        EMPTY      |");
	HELP_PUT_DASHED_LINE
	printf("\n| Supported Compile and Link command-line macros:                             |");
	printf("\n|    Short form          long form         expansion                          |");
	printf("\n|    \\fn                 \\filename         file's name with    extention      |");
	printf("\n|    \\bn                 \\basename         file's name without extention      |");
	printf("\n|    \\xt                 \\extention        file's extention                   |");
	printf("\n|    \\n                  \\newline          NEWLINE (\\n) followed by TAB (\\t)  |");
	printf("\n|    \\bs                 \\backslash        BACKSLASH (\\\\)                     |");
	HELP_PUT_DASHED_LINE
	getchar();
#endif
	
#ifdef	BORLANDC_DOS_VERSION
	textcolor( 7 );
	textbackground( 0 );
	cprintf("\n\r");
#endif
}
/*--------------------------------------------------------------------------*/
int	main(	unsigned short	params,
		char		*param[] )
{
	PUT_MAKEGEN_LOGO
	if ( options_read_profile( params, param ) != MMAKER_OK )
		return	MMAKER_NOT_OK;

	options_compute_general_variables();

	options_collect_sources();

	options_make_all( params, param );

	if ( options_save_setup_filename[ 0 ] != '\0' )
		options_save_setup( options_save_setup_filename );

	return MMAKER_OK;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#ifdef	BORLANDC_WINDOWS_VERSION
#include "mgenw.c_"		/* for WINDOWS' version source.		    */
#endif
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
