/*
 * xxmacros.c        -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xxmacros.h"
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
	MACRO_ENTRY	xxxx_macros[ MAX_MACROS_SUPPORTED ];
	int		xxxx_macros_counter = 0;
/*--------------------------------------------------------------------------*/
int	xxxx_macros_init_entry( int	index )
{
	if ( index >= MAX_MACROS_SUPPORTED )
		return 1;

	if ( index < 0 )
		return 2;

	xxxx_macros[ index ].name[ 0 ]		= '\0';
	xxxx_macros[ index ].ltrue_name[ 0 ]	= '\0';
	xxxx_macros[ index ].true_name[ 0 ]	= '\0';
	xxxx_macros[ index ].done		= 0;
	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_macros_init( void )
{
	int	i;

	xxxx_macros_counter = 0;
	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
		xxxx_macros_init_entry( i );
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_insert( char *name, char *temp_true_name )
{
	int	i;
	char	*dummy;

	if ( xxxx_macros_counter >= MAX_MACROS_SUPPORTED )
		return 1;

	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if ( xxxx_macros[ i ].name[ 0 ] == '\0' )
		{
			strncpy(xxxx_macros[ i ].name,
				name,
				MAX_MACRO_NAME - 1 );
			strncpy(xxxx_macros[ i ].ltrue_name,
				temp_true_name,
				MAX_MACRO_EXPANSION - 1 );
			strncpy(xxxx_macros[ i ].true_name,
				temp_true_name,
				MAX_MACRO_EXPANSION - 1 );
			xxxx_macros[ i ].done = 0;
			xxxx_macros_counter++;
			break;
		}
		else
			if ( xxxx_macros_is_in_database( name, &i ) )
			{
				strncpy(xxxx_macros[ i ].name,
					name,
					MAX_MACRO_NAME - 1 );
				strncpy(xxxx_macros[ i ].ltrue_name,
					temp_true_name,
					MAX_MACRO_EXPANSION - 1 );
				strncpy(xxxx_macros[ i ].true_name,
					temp_true_name,
					MAX_MACRO_EXPANSION - 1 );
				xxxx_macros[ i ].done = 0;
				break;
			}
	}

	if ( i == MAX_MACROS_SUPPORTED )
		return 2;

	if ( xxxx_macros_is_macro_within( xxxx_macros[ i ].true_name, &dummy ) )
		xxxx_macros[ i ].done = 0;
	else
		xxxx_macros[ i ].done = 1;
	return 0;
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_is_macro_within(	char	*str,
					char	**name )
{
	int	len = strlen( str );
	char	*ptr = str;

	while ( len && ( *ptr != '$' ))
	{
		ptr++;
		len--;
	}

	if ( len < 4 )
		return 0;

	ptr++;
	len--;

	if ( *ptr != '(' )
		return 0;

	ptr++;
	len--;
	*name = ptr - 2;
	ptr++;
	len--;

	while ( len && ( *ptr != ')' ))
	{
		ptr++;
		len--;
	}

	if ( *ptr != ')' )
		return 0;

	return 1;
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_this_macro_within(	char	*str,
					char	*macro_name,
					char	**name )
{
	char	search_str[ MAX_MACRO_EXPANSION ];
	char	*ptr;

	sprintf( search_str, "$(%s)", macro_name );
	ptr = strstr( str, search_str );
	if ( ptr != NULL )
	{
		*name = ptr;
		return 1;
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_is_in_database(	char	*name,
					int	*index	)
{
	int	i;

	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if ( strcmp( name, xxxx_macros[ i ].name ) == 0 )
		{
			*index = i;
			return 1;
		}
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_macros_print( FILE	*output )
{
	int	i;

	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if ( xxxx_macros[ i ].name[ 0 ] != '\0' )
		{
			fprintf(output,
				"\n%cm%s = %s",
				( i ? '+' : '-'),
				xxxx_macros[ i ].name,
				xxxx_macros[ i ].ltrue_name );
		}
	}
}
/*--------------------------------------------------------------------------*/
void	xxxx_macros_makefile_print( FILE	*output )
{
	int	i;

	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if ( xxxx_macros[ i ].name[ 0 ] != '\0' )
		{
			fprintf(output,
				"\n%s=%s",
				xxxx_macros[ i ].name,
				xxxx_macros[ i ].ltrue_name );
		}
	}
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_plant(	char	*name,
				char	*in_out_string,
				char	*data	)
{
	int	found = 0;
	int	len = strlen( name );
	char	copy[ MAX_MACRO_EXPANSION ];
	char	*macro_name_ptr;
	char	*rest_ptr;
	char	*in_out_ptr = in_out_string;

	while ( xxxx_macros_this_macro_within( in_out_string, name, &macro_name_ptr ) )
	{
		rest_ptr = macro_name_ptr + len + 3;
		macro_name_ptr[ 0 ]	  = '\0';
		macro_name_ptr[ len + 2 ] = '\0';
		sprintf(copy, "%s%s%s",
			in_out_string,
			data,
			rest_ptr );
		strcpy( in_out_ptr, copy );
		found++;
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_macros_merge( void )
{
	int	i;
	int	j;
	int	k;
	char	*dummy;

	for ( k = 0; k < MAX_MACROS_SUPPORTED; k++ )
	{
	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if (	( xxxx_macros[ i ].name[ 0 ]  != '\0' )	&&
			( xxxx_macros[ i ].done )		)
		{
			for ( j = 0; j < MAX_MACROS_SUPPORTED; j++ )
			{
				if (	( xxxx_macros[ j ].name[ 0 ]  != '\0' )	&&
					( !xxxx_macros[ j ].done )		)
				{
					xxxx_macros_plant(	xxxx_macros[ i ].name,
								xxxx_macros[ j ].true_name,
								xxxx_macros[ i ].true_name	);
					if ( !xxxx_macros_is_macro_within( xxxx_macros[ j ].true_name, &dummy ) )
						xxxx_macros[ j ].done = 1;
				}
			}
		}
	}
	}
	xxxx_macros_check_environment();
}
/*--------------------------------------------------------------------------*/
int	xxxx_macros_handle_line( char *line )
{
	int	sign;
	char	*name_ptr;
	char	*def_ptr;

	/*
	 * input line is of the following format:
	 *
	 *	{-|+}mmacro_name{\t| }={\t| }macro_expansion
	 */
	if ( ( *line != '-' ) && ( *line != '+' ) )
		return 1;
	sign = *line;
	line++;

	if ( *line != 'm' )
		return 2;
	line++;
	name_ptr = line;

	while ( ( *line != ' ' ) && ( *line != '\t' ) )
		line++;

	*line = '\0';
	line++;
	if ( *line != '=' )
		return 3;
	line++;
	*line = '\0';
	line++;
	def_ptr = line;

	if ( sign == (int)'-' )
		xxxx_macros_init();
	xxxx_macros_insert( name_ptr, def_ptr );

	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_macros_check_environment( void )
{
	int	i;
	int	j;
	int	cycles;
	int	max_cycles;
	char	min_str[] = "$(a)";
	char	*env_ptr;
	char	macro_name[ MAX_MACRO_NAME ];
	char	*macro_name_ptr;

	for ( i = 0; i < MAX_MACROS_SUPPORTED; i++ )
	{
		if (	( xxxx_macros[ i ].name[ 0 ] != '\0' )	&&
			(!xxxx_macros[ i ].done )		)
		{
		cycles = 0;
		max_cycles = strlen( xxxx_macros[ i ].ltrue_name ) / strlen( min_str );
		while ( xxxx_macros_is_macro_within(xxxx_macros[ i ].ltrue_name,
						 &macro_name_ptr	) &&
			( cycles < max_cycles ) )
		{
			cycles++;
			for ( j = 0; j < ( strlen( macro_name_ptr ) - 2 ); j++ )
			{
				macro_name[ j ] = *(macro_name_ptr + j + 2 );
			}
			macro_name[ j - 1 ] = '\0';
			env_ptr = getenv( macro_name_ptr );
			if ( env_ptr )
			{
				xxxx_macros_plant(macro_name_ptr,
						  xxxx_macros[ i ].ltrue_name,
						  env_ptr );
			}
		}
		if (!xxxx_macros_is_macro_within(xxxx_macros[ i ].ltrue_name,
						 &macro_name_ptr	) )
			xxxx_macros[ i ].done = 1;
		}
	}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
