/*
 * xxpathes.c        -	This file is part of the MakeGENerator program, a
 *			programmers' utility for automatic generation of
 *			makefiles.
 *
 * Copyright (C) 1997 Gil-ad Madar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Author: Gil-ad Madar
 *   e-mail: gil-ad@inter.net.il
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xxmacros.h"
#include "xxpathes.h"
#define PATH_MAX_LEN	1024
/*--------------------------------------------------------------------------*/
	extern MACRO_ENTRY	xxxx_macros[ MAX_MACROS_SUPPORTED ];
/*--------------------------------------------------------------------------*/
	PATH_ENTRY	xxxx_pathes[ MAX_PATHES_SUPPORTED ];
	int		xxxx_pathes_counter = 0;
	static char	private_path_str[ PATH_MAX_LEN ];
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_init_entry( int	index )
{
	if ( index >= MAX_PATHES_SUPPORTED )
		return 1;

	if ( index < 0 )
		return 2;

	xxxx_pathes[ index ].name		= NULL;
	xxxx_pathes[ index ].true_name[ 0 ]	= '\0';
	xxxx_pathes[ index ].done		= 0;
	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_pathes_init( void )
{
	int	i;

	xxxx_pathes_counter = 0;
	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
		xxxx_pathes_init_entry( i );
}
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_insert( char *name )
{
	int	i;
	char	*dummy;

	if ( xxxx_pathes_counter >= MAX_PATHES_SUPPORTED )
		return 1;

	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
	{
		if ( xxxx_pathes[ i ].name == NULL )
		{
			xxxx_pathes[ i ].name = name;
			strncpy(xxxx_pathes[ i ].true_name,
				name,
				MAX_PATH_EXPANSION - 1 );
			xxxx_pathes[ i ].done = 0;
			xxxx_pathes_counter++;
			break;
		}
	}

	if ( i == MAX_PATHES_SUPPORTED )
		return 2;

	if ( !xxxx_macros_is_macro_within( xxxx_pathes[ i ].true_name, &dummy ) )
		xxxx_pathes[ i ].done = 1;
	return 0;
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_this_path_within(	char	*str,
					char	*path_name,
					char	**name )
{
	char	search_str[ MAX_PATH_EXPANSION ];
	char	*ptr;

	sprintf( search_str, "$(%s)", path_name );
	ptr = strstr( str, search_str );
	if ( ptr != NULL )
	{
		*name = ptr;
		return 1;
	}

	return 0;
}
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_read_path_str( char	*path_str )
{
	int	i;
	int	j;
	int	len = strlen( path_str );


	if ( path_str == NULL )
		return 1;

	if ( path_str[ 0 ] == '\0' )
		return 2;

	strcpy( private_path_str, path_str );
	for ( i = 0; i < len; i++ )
	{
		if ( private_path_str[ i ] == ';' )
			private_path_str[ i ] = '\0';
	}

	i = 0;
	j = 0;
	while (	( j < MAX_PATHES_SUPPORTED )	&&
		( i < len )				)
	{
		xxxx_pathes_insert( private_path_str + i );
		i += strlen( private_path_str + i ) + 1;
		j++;
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_pathes_merge( void )
{
	int	i;
	int	j;
	char	*dummy;

	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
	{
		if (	( xxxx_pathes[ i ].name != NULL )	&&
			( !xxxx_pathes[ i ].done )		)
		{
			for ( j = 0; j < MAX_MACROS_SUPPORTED; j++ )
			{
				if (	( xxxx_macros[ j ].name[ 0 ]  != '\0' )	&&
					( xxxx_macros[ j ].done )		)
				{
					xxxx_macros_plant(	xxxx_macros[ j ].name,
								xxxx_pathes[ i ].true_name,
								xxxx_macros[ j ].true_name	);
					if ( !xxxx_macros_is_macro_within( xxxx_pathes[ i ].true_name, &dummy ) )
						xxxx_pathes[ i ].done = 1;
				}
			}
		}
	}
	xxxx_pathes_check_environment();
}
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_write_path_str( char *path_str )
{
	int	i;
	char	true_name[ MAX_PATH_EXPANSION ];


	if ( path_str == NULL )
		return 1;

	path_str[ 0 ] = '\0';

	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
	{
		if (	( xxxx_pathes[ i ].name != NULL )	&&
			(*xxxx_pathes[ i ].name != '\0' )	)
		{
			sprintf( true_name, "%s;", xxxx_pathes[ i ].true_name );
			strcat( path_str, true_name );
		}
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
int	xxxx_pathes_write_original_path_str( char *path_str )
{
	int	i;
	char	true_name[ MAX_PATH_EXPANSION ];


	if ( path_str == NULL )
		return 1;

	path_str[ 0 ] = '\0';

	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
	{
		if (	( xxxx_pathes[ i ].name != NULL  )	&&
			( *xxxx_pathes[ i ].name != '\0' )	)
		{
			sprintf( true_name, "%s;", xxxx_pathes[ i ].name );
			strcat( path_str, true_name );
		}
	}
	return 0;
}
/*--------------------------------------------------------------------------*/
void	xxxx_pathes_check_environment( void )
{
	int	i;
	int	j;
	int	cycles;
	int	max_cycles;
	char	min_str[] = "$(a)";
	char	*env_ptr;
	char	macro_name[ MAX_PATH_NAME ];
	char	*macro_name_ptr;

	for ( i = 0; i < MAX_PATHES_SUPPORTED; i++ )
	{
		if (	( xxxx_pathes[ i ].name != NULL )	&&
			(!xxxx_pathes[ i ].done )		)
		{
		cycles = 0;
		max_cycles = strlen( xxxx_pathes[ i ].true_name ) / strlen( min_str );
		while ( xxxx_macros_is_macro_within(xxxx_pathes[ i ].true_name,
						 &macro_name_ptr	) &&
			( cycles < max_cycles ) )
		{
			cycles++;
			for ( j = 0; j < ( strlen( macro_name_ptr ) - 2 ); j++ )
			{
				macro_name[ j ] = *(macro_name_ptr + j + 2 );
			}
			macro_name[ j - 1 ] = '\0';
			env_ptr = getenv( macro_name );
			if ( env_ptr )
			{
				xxxx_macros_plant(macro_name,
						  xxxx_pathes[ i ].true_name,
						  env_ptr );
			}
		}
		if (!xxxx_macros_is_macro_within(xxxx_pathes[ i ].true_name,
						 &macro_name_ptr	) )
			xxxx_pathes[ i ].done = 1;
		}
	}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
